library IEEE;
library work;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

--! Packages from IP cores
--use work.wishbone_pkg.ALL;
use work.image1_pkg.ALL;
use work.rtm_detector_pkg.ALL;
use work.i2c_slave_pkg.ALL;
use work.m25p32_pkg.ALL;
use work.multiboot_pkg.ALL;

package image1_top_tb_pkg is

   constant c_SCL_I2C_DRIVER_PERIOD  : TIME := 10 us;

   constant c_NUMBER_OF_CHANNELS : NATURAL
                                       := work.image1_pkg.c_NUMBER_OF_CHANNELS;

   component image1_top
   -- generic(g_NUMBER_OF_CHANNELS : NATURAL := 6);
   port (RST_N                : in  STD_LOGIC;
         CLK20_VCXO           : in  STD_LOGIC;
         FPGA_CLK_P           : in  STD_LOGIC; --Using the 125MHz clock
         FPGA_CLK_N           : in  STD_LOGIC;
         --! LEDs
         LED_CTRL0            : out STD_LOGIC;
         LED_CTRL0_OEN        : out STD_LOGIC;
         LED_CTRL1            : out STD_LOGIC;
         LED_CTRL1_OEN        : out STD_LOGIC;
         LED_MULTICAST_2_0    : out STD_LOGIC;
         LED_MULTICAST_3_1    : out STD_LOGIC;
         LED_WR_GMT_TTL_TTLN  : out STD_LOGIC;
         LED_WR_LINK_SYSERROR : out STD_LOGIC;
         LED_WR_OK_SYSPW      : out STD_LOGIC;
         LED_WR_OWNADDR_I2C   : out STD_LOGIC;
         --! I/Os for pulses
         PULSE_FRONT_LED_N    : out STD_LOGIC_VECTOR(6 downto 1);
         PULSE_REAR_LED_N     : out STD_LOGIC_VECTOR(6 downto 1);
         FPGA_INPUT_TTL_N     : in  STD_LOGIC_VECTOR(6 downto 1);
         FPGA_OUT_TTL         : out STD_LOGIC_VECTOR(6 downto 1);
         FPGA_BLO_IN          : in  STD_LOGIC_VECTOR(6 downto 1);
         FPGA_TRIG_BLO        : out STD_LOGIC_VECTOR(6 downto 1);
         INV_IN_N             : in  STD_LOGIC_VECTOR(4 downto 1);
         INV_OUT              : out STD_LOGIC_VECTOR(4 downto 1);
         --! Lines for the i2c_slave
         SCL_I                : in  STD_LOGIC;
         SCL_O                : out STD_LOGIC;
         SCL_OE               : out STD_LOGIC;
         SDA_I                : in  STD_LOGIC;
         SDA_O                : out STD_LOGIC;
         SDA_OE               : out STD_LOGIC;
         FPGA_GA              : in  STD_LOGIC_VECTOR(4 downto 0);
         FPGA_GAP             : in  STD_LOGIC;
         --! Pins of the SPI interface to write into the Flash memory
         FPGA_PROM_CCLK       : out STD_LOGIC;
         FPGA_PROM_CSO_B_N    : out STD_LOGIC;
         FPGA_PROM_DIN        : in  STD_LOGIC;
         FPGA_PROM_MOSI       : out STD_LOGIC;
         --! RTM identifiers, should match with the expected values
         --! TODO: add matching
         FPGA_OE              : out STD_LOGIC;
         FPGA_BLO_OE          : out STD_LOGIC;
         FPGA_TRIG_TTL_OE     : out STD_LOGIC;
         FPGA_INV_OE          : out STD_LOGIC;
         LEVEL                : in  STD_LOGIC;
         EXTRA_SWITCH         : in  STD_LOGIC_VECTOR(1 downto 1);--! General enable
         MR_N                 : out STD_LOGIC;--! It allows power sequencing of the
                                              --! 24V rail after a security given
                                              --! delay
         FPGA_RTMM_N_i          : in STD_LOGIC_VECTOR(2 downto 0);
         FPGA_RTMP_N_i          : in STD_LOGIC_VECTOR(2 downto 0));
   end component;

   type t_pulse_vector is
      record
         FRONT : STD_LOGIC_VECTOR(c_NUMBER_OF_CHANNELS - 1 downto 0);
         REAR  : STD_LOGIC_VECTOR(c_NUMBER_OF_CHANNELS - 1 downto 0);
      end record;
   subtype t_pulse_led_vector is t_pulse_vector;

   type t_fpga_en is
      record
         GEN    : STD_LOGIC;
         BLO    : STD_LOGIC;
         TTL    : STD_LOGIC;
         INV    : STD_LOGIC;
      end record;

   type t_I2C_master_out is
      record
         SCL  : STD_LOGIC;
         SDA  : STD_LOGIC;
      end record;
   subtype t_I2C_slave_in is t_I2C_master_out;

   type t_I2C_master_in is
      record
         SDA  : STD_LOGIC;
      end record;
   subtype t_I2C_slave_out is t_I2C_master_in;

   type t_SPI_master_out is
      record
         CCLK    : STD_LOGIC;
         CSO_B_N : STD_LOGIC;
         MOSI    : STD_LOGIC;
      end record;
   subtype t_SPI_slave_in is t_SPI_master_out;

   type t_SPI_master_in is
      record
         DIN     : STD_LOGIC;
      end record;
   subtype t_SPI_slave_out is t_SPI_master_in;

   type t_RTM_id is
      record
         RTMM : STD_LOGIC_VECTOR(2 downto 0);
         RTMP : STD_LOGIC_VECTOR(2 downto 0);
      end record;

   type t_I2C_regs is
      record
         CTR0 : work.i2c_slave_pkg.r_CTR0;
         LT   : work.i2c_slave_pkg.r_LT;
         DRXA : STD_LOGIC_VECTOR(work.i2c_slave_pkg.r_DRX'a_length - 1 downto 0);
         DRXB : STD_LOGIC_VECTOR(work.i2c_slave_pkg.r_DRX'a_length - 1 downto 0);
         DTX  : STD_LOGIC_VECTOR(work.i2c_slave_pkg.r_DTX'a_length - 1 downto 0);
      end record;

   type t_MULTIBOOT_regs is
      record
        CTR0     : work.multiboot_pkg.r_CTR0;
        CTR1     : work.multiboot_pkg.r_CTR1;
        STAT     : work.multiboot_pkg.r_STAT;
        MBA      : work.multiboot_pkg.r_BAR;
        GBA      : work.multiboot_pkg.r_BAR;
        MBA_ICAP : STD_LOGIC_VECTOR (23 downto 0);
        GBA_ICAP : STD_LOGIC_VECTOR (23 downto 0);
      end record;

   type t_MULTIBOOT_regs_slv is
      record
        CTR0     : STD_LOGIC_VECTOR (31 downto 0);
        CTR1     : STD_LOGIC_VECTOR (31 downto 0);
        STAT     : STD_LOGIC_VECTOR (31 downto 0);
        MBA      : STD_LOGIC_VECTOR (31 downto 0);
        GBA      : STD_LOGIC_VECTOR (31 downto 0);
        MBA_ICAP : STD_LOGIC_VECTOR (31 downto 0);
        GBA_ICAP : STD_LOGIC_VECTOR (31 downto 0);
      end record;

   type t_M25P32_regs is
      record
         FMI       : work.m25p32_pkg.r_FMI;
         SR_m25p32 : work.m25p32_pkg.r_SR_m25p32;
      end record;

   type t_M25P32_regs_slv is
      record
         FMI       : STD_LOGIC_VECTOR(31 downto 0);
         SR_m25p32 : STD_LOGIC_VECTOR(31 downto 0);
      end record;

   type t_wr_fw_reg is
      record
         I2C_CTR0               : work.i2c_slave_pkg.r_CTR0;
         MULTIBOOT_CTR0         : work.multiboot_pkg.r_CTR0;
         MULTIBOOT_MBA          : work.multiboot_pkg.r_BAR;
         MULTIBOOT_GBA          : work.multiboot_pkg.r_BAR;
         M25P32_FMI             : work.m25p32_pkg.r_FMI;
         M25P32_SR_m25p32       : work.m25p32_pkg.r_SR_m25p32;
         M25P32_DATA_WRITE      : STD_LOGIC_VECTOR(31 downto 0);
      end record;

   type t_wr_fw_reg_slv is
      record
         I2C_CTR0               : STD_LOGIC_VECTOR(31 downto 0);
         MULTIBOOT_CTR0         : STD_LOGIC_VECTOR(31 downto 0);
         MULTIBOOT_MBA          : STD_LOGIC_VECTOR(31 downto 0);
         MULTIBOOT_GBA          : STD_LOGIC_VECTOR(31 downto 0);
         M25P32_FMI             : STD_LOGIC_VECTOR(31 downto 0);
         M25P32_SR_m25p32       : STD_LOGIC_VECTOR(31 downto 0);
         M25P32_DATA_WRITE      : STD_LOGIC_VECTOR(31 downto 0);
      end record;


   constant c_fpga_en_default  : t_fpga_en := (GEN => '0',
                                               BLO => '0',
                                               TTL => '0',
                                               INV => '0');

   constant c_I2C_master_out_default : t_I2C_master_out := (SCL  => '1',
                                                            SDA  => '1');

   constant c_RTM_id_default : t_RTM_id
                         := (RTMM => STD_LOGIC_VECTOR(c_RTMM_V1),
                             RTMP => STD_LOGIC_VECTOR(c_RTMP_BLOCKING_V1));

   --! ==================================
   --! MEMORY MAPPINGS
   --! ==================================
   --! Byte aligned
   --! It should be noted that all internal modules are word aligned.
   --! Thus, access to byte 1,2 or 3 within an internal word register
   --! will be processed as an access to such register (indeed, like
   --! accessing to byte 0).
   --! ==================================
   --!    MULTIBOOT   [0200-02FF]
   --!    M25P32      [0080-00BF]
   --!    I2C_SLAVE   [0040-007F]
   --! ==================================
   --! c_I2C_CTR0_addr             [0040]
   --! c_I2C_LT_addr               [0044]
   --! c_I2C_DTX_addr              [0048]
   --! c_I2C_DRXA_addr             [004C]
   --! c_I2C_DRXB_addr             [0050]
   --! c_MULTIBOOT_CTR0_addr       [0080] W
   --! c_MULTIBOOT_CTR1_addr       [0084]
   --! c_MULTIBOOT_STAT_addr       [0088]
   --! c_MULTIBOOT_MBA_addr        [008C] W
   --! c_MULTIBOOT_GBA_addr        [0090] W
   --! c_MULTIBOOT_MBA_ICAP_addr   [0094]
   --! c_MULTIBOOT_GBA_ICAP_addr   [0098]
   --! c_M25P32_FMI_addr           [0200] W
   --! c_M25P32_SR_m25p32_addr     [0204] W
   --! c_M25P32_DATA_READ_addr     [0208]
   --! c_M25P32_DATA_WRITE_addr    [0300 - 03FF] W
   --! ==================================

   constant c_I2C_CTR0_addr                    : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"01"& STD_LOGIC_VECTOR(work.i2c_slave_pkg.c_CTR0_addr) & "00";
   constant c_I2C_LT_addr                      : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"01"& STD_LOGIC_VECTOR(work.i2c_slave_pkg.c_LT_addr)   & "00";
   constant c_I2C_DTX_addr                     : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"01"& STD_LOGIC_VECTOR(work.i2c_slave_pkg.c_DTX_addr)  & "00";
   constant c_I2C_DRXA_addr                    : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"01"& STD_LOGIC_VECTOR(work.i2c_slave_pkg.c_DRXA_addr) & "00";
   constant c_I2C_DRXB_addr                    : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"01"& STD_LOGIC_VECTOR(work.i2c_slave_pkg.c_DRXB_addr) & "00";
   constant c_MULTIBOOT_CTR0_addr              : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"10"& STD_LOGIC_VECTOR(work.multiboot_pkg.CTR0_addr) &"00";
   constant c_MULTIBOOT_CTR1_addr              : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"10"& STD_LOGIC_VECTOR(work.multiboot_pkg.CTR1_addr) &"00";
   constant c_MULTIBOOT_STAT_addr              : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"10"& STD_LOGIC_VECTOR(work.multiboot_pkg.STAT_addr) &"00";
   constant c_MULTIBOOT_MBA_addr               : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"10"& STD_LOGIC_VECTOR(work.multiboot_pkg.c_MBA_addr) &"00";
   constant c_MULTIBOOT_GBA_addr               : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"10"& STD_LOGIC_VECTOR(work.multiboot_pkg.c_GBA_addr) &"00";
   constant c_MULTIBOOT_MBA_ICAP_addr          : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"10"& STD_LOGIC_VECTOR(work.multiboot_pkg.c_MBA_ICAP_addr) &"00";
   constant c_MULTIBOOT_GBA_ICAP_addr          : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"10"& STD_LOGIC_VECTOR(work.multiboot_pkg.c_GBA_ICAP_addr) &"00";
   constant c_M25P32_FMI_addr                  : STD_LOGIC_VECTOR(15 downto 0)
      := X"0"&"001"& work.m25p32_pkg.c_FMI_addr        &"00";
   constant c_M25P32_SR_m25p32_addr            : STD_LOGIC_VECTOR(15 downto 0)
      := X"0"&"001"& work.m25p32_pkg.c_SR_m25p32_addr  &"00";
   constant c_M25P32_DATA_READ_addr            : STD_LOGIC_VECTOR(15 downto 0)
      := X"0"&"001"& work.m25p32_pkg.c_DATA_READ_addr  &"00";
   constant c_M25P32_DATA_WRITE_addr           : STD_LOGIC_VECTOR(15 downto 0)
      := X"0"&"001"& work.m25p32_pkg.c_DATA_WRITE_addr &"00";
      constant c_M25P32_DATA_WRITE_base_addr   : UNSIGNED(15 downto 0)
                                       := UNSIGNED(c_M25P32_DATA_WRITE_addr);

   function f_STD_LOGIC_VECTOR(fw_regs : t_wr_fw_reg) return t_wr_fw_reg_slv;

   function f_STD_LOGIC_VECTOR(multiboot_regs : t_MULTIBOOT_regs)
                                                 return t_MULTIBOOT_regs_slv;

   function f_STD_LOGIC_VECTOR(m25p32_regs : t_M25P32_regs)
                                                 return t_M25P32_regs_slv;


end image1_top_tb_pkg;

package body image1_top_tb_pkg is

   --!                        Translation function to help checking within
   --!                        image1_top_tb
   --! @brief multiboot_regs  Aggregate of the register in multiboot core
   function f_STD_LOGIC_VECTOR(fw_regs : t_wr_fw_reg) return t_wr_fw_reg_slv is
      variable v_return  : t_wr_fw_reg_slv;
   begin
      v_return.I2C_CTR0                := f_STD_LOGIC_VECTOR(fw_regs.I2C_CTR0);
      v_return.MULTIBOOT_CTR0          := X"000000"
                                            & f_STD_LOGIC_VECTOR(fw_regs.MULTIBOOT_CTR0);
      v_return.MULTIBOOT_MBA           := f_STD_LOGIC_VECTOR(fw_regs.MULTIBOOT_MBA);
      v_return.MULTIBOOT_GBA           := f_STD_LOGIC_VECTOR(fw_regs.MULTIBOOT_GBA);
      v_return.M25P32_FMI              := X"00"
                                            & f_STD_LOGIC_VECTOR(fw_regs.M25P32_FMI);
      v_return.M25P32_SR_m25p32        := X"000000"
                                            & f_STD_LOGIC_VECTOR(fw_regs.M25P32_SR_m25p32);
      v_return.M25P32_DATA_WRITE       := fw_regs.M25P32_DATA_WRITE;
      return v_return;
   end f_STD_LOGIC_VECTOR;

   --!                        Translation function to help checking within
   --!                        image1_top_tb
   --! @brief multiboot_regs  Aggregate of the register in multiboot core
   function f_STD_LOGIC_VECTOR(multiboot_regs : t_MULTIBOOT_regs)
                                                 return t_MULTIBOOT_regs_slv is
      variable v_return  : t_MULTIBOOT_regs_slv;
   begin
      v_return.CTR0     := X"000000"&f_STD_LOGIC_VECTOR(multiboot_regs.CTR0);
      v_return.CTR1     := X"000000"&f_STD_LOGIC_VECTOR(multiboot_regs.CTR1);
      v_return.STAT     := X"0000"&f_STD_LOGIC_VECTOR(multiboot_regs.STAT);
      v_return.MBA      := f_STD_LOGIC_VECTOR(multiboot_regs.MBA);
      v_return.GBA      := f_STD_LOGIC_VECTOR(multiboot_regs.GBA);
      v_return.MBA_ICAP := X"00"&multiboot_regs.MBA_ICAP;
      v_return.GBA_ICAP := X"00"&multiboot_regs.GBA_ICAP;
      return v_return;
   end function;

   --!                     Translation function to help checking within
   --!                     image1_top_tb
   --! @brief m25p32_regs  Aggregate of the register in m25p32 core
   function f_STD_LOGIC_VECTOR(m25p32_regs : t_M25P32_regs)
                                                    return t_M25P32_regs_slv is
      variable v_return  : t_M25P32_regs_slv;
   begin
      v_return.FMI       := X"00"&f_STD_LOGIC_VECTOR(m25p32_regs.FMI);
      v_return.SR_m25p32 := X"000000"&f_STD_LOGIC_VECTOR(m25p32_regs.SR_m25p32);
      return v_return;
   end function;

end image1_top_tb_pkg;
