--==============================================================================
-- CERN (BE-CO-HT)
-- Testbench for old repeater design
--==============================================================================
--
-- author: Theodor Stana (t.stana@cern.ch)
--
-- date of creation: 2013-02-28
--
-- version: 1.0
--
-- description: 
--
-- dependencies:
--
-- references:
-- 
--==============================================================================
-- GNU LESSER GENERAL PUBLIC LICENSE
--==============================================================================
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
--==============================================================================
-- last changes:
--    2013-02-28   Theodor Stana     t.stana@cern.ch     File created
--==============================================================================
-- TODO: - 
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


entity testbench is
end entity testbench;


architecture behav of testbench is

  --============================================================================
  -- Type declarations
  --============================================================================

  --============================================================================
  -- Constant declarations
  --============================================================================
  constant c_clk_per : time := 10 ns;
  constant c_reset_width : time := 31 ns;

  --============================================================================
  -- Component declarations
  --============================================================================

  component pulse_gen is
    generic
    (
      g_pwidth : natural := 200;
      g_freq   : natural := 200000000;
      g_delay  : natural := 1
    );
    port
    (
      clk_i   : in  std_logic;
      rst_n_i : in  std_logic;
      en_i    : in  std_logic;
      pulse_o : out std_logic
    );
  end component pulse_gen;

  --============================================================================
  -- Signal declarations
  --============================================================================
  signal clk, rst_n     : std_logic := '0';
  signal pulse0, pulse1, pulse2 : std_logic := '0';


--==============================================================================
--  architecture begin
--==============================================================================
begin

  DUT0: pulse_gen
  generic map
  (
    g_pwidth => 1,
    g_freq   => 20,
    g_delay  => 0
  )
  port map
  (
    clk_i   => clk,
    rst_n_i => rst_n,
    en_i    => '1',
    pulse_o => pulse0
  );

  DUT1: pulse_gen
  generic map
  (
    g_pwidth => 1,
    g_freq   => 20,
    g_delay  => 3
  )
  port map
  (
    clk_i   => clk,
    rst_n_i => rst_n,
    en_i    => '1',
    pulse_o => pulse1
  );

  DUT2: pulse_gen
  generic map
  (
    g_pwidth => 1,
    g_freq   => 20,
    g_delay  => 8
  )
  port map
  (
    clk_i   => clk,
    rst_n_i => rst_n,
    en_i    => '1',
    pulse_o => pulse2
  );

  p_clk: process
  begin
    clk <= not clk;
    wait for c_clk_per/2;
  end process p_clk;

  p_rst: process
  begin
    rst_n <= '0';
    wait for c_reset_width;
    rst_n <= '1';
    wait;
  end process p_rst;

end architecture behav;
--==============================================================================
--  architecture end
--==============================================================================
