library IEEE;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

--! Code intended for simulation only.

entity wishbone_driver is
   generic(g_WB_DATA_O_LENGTH : NATURAL := 32;
           g_WB_DATA_I_LENGTH : NATURAL := 32;
           g_WB_ADDR_LENGTH   : NATURAL := 6;
           g_WB_CLK_PERIOD : TIME    := 50 ns);
   port(wb_clk_i : in STD_LOGIC;
        wb_rst_i : in STD_LOGIC;

        wb_stb_o  : out STD_LOGIC;
        wb_cyc_o  : out STD_LOGIC;
        wb_sel_o  : out STD_LOGIC_VECTOR(3  downto 0);
        wb_we_o   : out STD_LOGIC;
        wb_data_i : in  STD_LOGIC_VECTOR(g_WB_DATA_I_LENGTH - 1 downto 0);
        wb_data_o : out STD_LOGIC_VECTOR(g_WB_DATA_O_LENGTH - 1 downto 0);
        wb_addr_o : out STD_LOGIC_VECTOR(g_WB_ADDR_LENGTH   - 1 downto 0);
        wb_ack_i  : in  STD_LOGIC;
        wb_rty_i  : in  STD_LOGIC;
        wb_err_i  : in  STD_LOGIC;

        data_i    : in  STD_LOGIC_VECTOR(g_WB_DATA_O_LENGTH - 1 downto 0);
        data_o    : out STD_LOGIC_VECTOR(g_WB_DATA_I_LENGTH - 1 downto 0);
        addr_i    : in  STD_LOGIC_VECTOR(g_WB_ADDR_LENGTH   - 1 downto 0);

        write_i          : in  STD_LOGIC;
        write_done_o     : out STD_LOGIC;
        read_i           : in  STD_LOGIC;
        read_done_o      : out STD_LOGIC);
   end  wishbone_driver;

architecture Behavioral of wishbone_driver is


begin

   p_read_write: process
   begin
      if read_i = '1' then
           wait until rising_edge(wb_clk_i);
         wb_we_o    <= '0';
         wb_stb_o   <= '1';
         wb_cyc_o   <= '1';
         wb_sel_o   <= (others => '1');
         wb_addr_o  <= addr_i;
           wait until rising_edge(wb_clk_i);
           wait until wb_ack_i'event;
           --! Here we wait for the ack and keep till operation is finished
         data_o     <= wb_data_i;
           wait until rising_edge(wb_clk_i);
         wb_stb_o    <= '0';
         wb_cyc_o    <= '0';
         wb_sel_o    <= (others => '0');
         read_done_o <= '1';
      elsif write_i = '1' then
           wait until rising_edge(wb_clk_i);
         wb_we_o    <= '1';
         wb_stb_o   <= '1';
         wb_cyc_o   <= '1';
         wb_sel_o   <= (others => '1');
         wb_data_o  <= data_i;
         wb_addr_o  <= addr_i;
           wait until rising_edge(wb_clk_i);
           --! Here we wait for the ack and keep till operation is finished
           wait until rising_edge(wb_clk_i);
         wb_we_o    <= '0';
         wb_stb_o   <= '0';
         wb_cyc_o   <= '0';
         wb_sel_o   <= (others => '0');
         write_done_o <= '1';
      else
         wb_stb_o     <= '0';
         wb_cyc_o     <= '0';
         wb_sel_o     <= (others => '0');
         wb_we_o      <= '0';
         wb_data_o    <= (others => '0');
         wb_addr_o    <= (others => '0');
         data_o       <= (others => '0');
         write_done_o <= '0';
         read_done_o  <= '0';
      end if;
      wait until rising_edge(wb_clk_i);
   end process;

end;
